<?php
/*
 * NagCollect -- Nagios Data Collector for Passive Checks
 * Copyright (c)2009-2010 Alexander Barton, alex@barton.de
 */

function nagiosSubmitHost($host, $status, $text = null)
{
	$params  = escapeshellarg($host);
	$params .= ' ' . escapeshellarg($status);
	if ($text)
		$params .= ' ' . escapeshellarg($text);
	$cmd = 'sudo -u nagios /usr/local/sbin/nagios-submit-host -c ' . $params;
	exec($cmd, $output, $result);
	if ($result != 0)
		error_log("NagCollect: nagios-submit-host=$result: " . $output[0]);
	return ($result == 0);
}

function nagiosSubmitService($host, $service, $status, $text = null)
{
	$params  = escapeshellarg($host);
	$params .= ' ' . escapeshellarg($service);
	$params .= ' ' . escapeshellarg($status);
	if ($text)
		$params .= ' ' . escapeshellarg($text);
	$cmd = 'sudo -u nagios /usr/local/sbin/nagios-submit-service -c ' . $params;
	exec($cmd, $output, $result);
	if ($result != 0)
		error_log("NagCollect: nagios-submit-service=$result: " . $output[0]);
	return ($result == 0);
}

function checkKey($key, $host)
{
	$fh = @fopen('/etc/nagios3/nagcollect.keys', 'r');
	if (!$fh) {
		error_log("NagCollect: can't open \"/etc/nagios3/nagcollect.keys\"!");
		return false;
	}
	while ($str = fgets($fh, 1024)) {
		$str = trim($str);
		if (!$str || $str[0] == '#' || $str[0] == '/')
			continue;
		$p = strpos($str, ':');
		if ($p)
			$str = trim(substr($str, 0, $p));
		if ($str == $key) {
			fclose($fh);
			return true;
		}
	}
	fclose($fh);
	return false;
}

function processRequest($args)
{
	// Check that this is a valid (POST-) request
	if (!isset($args['host']) || !isset($args['status']))
		return 400;

	$host = $args['host'];
	$status = $args['status'];

	// Make sure the authorizsation key is correct
	if (!isset($args['key']) || !checkKey($args['key'], $host))
		return 401;

	$service = isset($args['service']) ? $args['service'] : null;
	$text = isset($args['text']) ? $args['text'] : null;

	if (!$service) {
		// Host Update
		if (!nagiosSubmitHost($host, $status, $text))
			return 500;
	} else {
		// Service Update
		$hostStatus = 'Received passive service check from '
					. '"' . $_SERVER['REMOTE_ADDR'] . '"';
		if (!nagiosSubmitHost($host, 0, $hostStatus))
			return 500;
		if (!nagiosSubmitService($host, $service, $status, $text))
			return 500;
	}
	return 200;
}

function getHttpStatusText($status)
{
	switch ($status) {
		case 200:	return "OK";
		case 400:	return "Bad Request";
		case 401:	return "Unauthorized";
		case 500:	return "Internal Server Error";
		default:	return "Unknown";
	}
}

openlog("NagCollect", LOG_ODELAY, LOG_DAEMON);

$httpStatus = processRequest($_POST);
$httpStatusText = getHttpStatusText($httpStatus);

$statusText = $httpStatus . ' - ' . $httpStatusText;

if ($httpStatus != 200) {
	syslog(LOG_WARNING, "Warning: $httpStatusText ($httpStatus) from "
		   . "\"{$_SERVER['REMOTE_ADDR']}\" ({$_SERVER['HTTP_USER_AGENT']})");
}

closelog();

header("HTTP/1.0 $httpStatus $httpStatusText");

?><!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01//EN"
	"http://www.w3.org/TR/html4/strict.dtd">
<html lang="en">
<head>
	<meta http-equiv="Content-type" content="text/html; charset=utf-8">
	<title>NagCollect: <?php echo $statusText ?></title>
</head>
<body id="nagcollect">
<h1><?php echo $statusText ?></h1>
</body>
</html>
